@@ -3,9 +3,7 @@
 Package containing implementation of all the standard Distutils
 commands."""
 
-# This module should be kept compatible with Python 2.1.
-
-__revision__ = "$Id$"
+__revision__ = "$Id: __init__.py 57699 2007-08-30 03:52:21Z collin.winter $"
 
 __all__ = ['build',
            'build_py',
@@ -24,8 +22,6 @@ __all__ = ['build',
            'bdist_dumb',
            'bdist_rpm',
            'bdist_wininst',
-           'upload',
-
            # These two are reserved for future use:
            #'bdist_sdux',
            #'bdist_pkgtool',
