@@ -4,8 +4,9 @@ Contains MWerksCompiler, an implementation of the abstract CCompiler class
 for MetroWerks CodeWarrior on the Macintosh. Needs work to support CW on
 Windows."""
 
-import sys, os, string
-from types import *
+__revision__ = "$Id: mwerkscompiler.py 65250 2008-07-26 20:09:45Z amaury.forgeotdarc $"
+
+import sys, os
 from distutils.errors import \
      DistutilsExecError, DistutilsPlatformError, \
      CompileError, LibError, LinkError
@@ -13,10 +14,10 @@ from distutils.ccompiler import \
      CCompiler, gen_preprocess_options, gen_lib_options
 import distutils.util
 import distutils.dir_util
-import mkcwproject
+from distutils import log
 
 class MWerksCompiler (CCompiler) :
-    """Concrete class that implements an interface to Microsoft Visual C++,
+    """Concrete class that implements an interface to MetroWerks CodeWarrior,
        as defined by the CCompiler abstract class."""
 
     compiler_type = 'mwerks'
@@ -52,8 +53,8 @@ class MWerksCompiler (CCompiler) :
                   force=0):
 
         CCompiler.__init__ (self, verbose, dry_run, force)
-        
-        
+
+
     def compile (self,
                  sources,
                  output_dir=None,
@@ -61,12 +62,16 @@ class MWerksCompiler (CCompiler) :
                  include_dirs=None,
                  debug=0,
                  extra_preargs=None,
-                 extra_postargs=None):
-         self.__sources = sources
-         self.__macros = macros
-         self.__include_dirs = include_dirs
-         # Don't need extra_preargs and extra_postargs for CW
-         
+                 extra_postargs=None,
+                 depends=None):
+        (output_dir, macros, include_dirs) = \
+           self._fix_compile_args (output_dir, macros, include_dirs)
+        self.__sources = sources
+        self.__macros = macros
+        self.__include_dirs = include_dirs
+        # Don't need extra_preargs and extra_postargs for CW
+        return []
+
     def link (self,
               target_desc,
               objects,
@@ -79,24 +84,30 @@ class MWerksCompiler (CCompiler) :
               debug=0,
               extra_preargs=None,
               extra_postargs=None,
-              build_temp=None):
+              build_temp=None,
+              target_lang=None):
+        # First fixup.
+        (objects, output_dir) = self._fix_object_args (objects, output_dir)
+        (libraries, library_dirs, runtime_library_dirs) = \
+            self._fix_lib_args (libraries, library_dirs, runtime_library_dirs)
+
         # First examine a couple of options for things that aren't implemented yet
         if not target_desc in (self.SHARED_LIBRARY, self.SHARED_OBJECT):
-            raise DistutilsPlatformError, 'Can only make SHARED_LIBRARY or SHARED_OBJECT targets on the Mac'
+            raise DistutilsPlatformError('Can only make SHARED_LIBRARY or SHARED_OBJECT targets on the Mac')
         if runtime_library_dirs:
-            raise DistutilsPlatformError, 'Runtime library dirs not implemented yet'
+            raise DistutilsPlatformError('Runtime library dirs not implemented yet')
         if extra_preargs or extra_postargs:
-            raise DistutilsPlatformError, 'Runtime library dirs not implemented yet'
+            raise DistutilsPlatformError('Runtime library dirs not implemented yet')
         if len(export_symbols) != 1:
-            raise DistutilsPlatformError, 'Need exactly one export symbol'
+            raise DistutilsPlatformError('Need exactly one export symbol')
         # Next there are various things for which we need absolute pathnames.
         # This is because we (usually) create the project in a subdirectory of
         # where we are now, and keeping the paths relative is too much work right
         # now.
-        sources = map(self._filename_to_abs, self.__sources)
-        include_dirs = map(self._filename_to_abs, self.__include_dirs)
+        sources = [self._filename_to_abs(s) for s in self.__sources]
+        include_dirs = [self._filename_to_abs(d) for d in self.__include_dirs]
         if objects:
-            objects = map(self._filename_to_abs, objects)
+            objects = [self._filename_to_abs(o) for o in objects]
         else:
             objects = []
         if build_temp:
@@ -114,6 +125,8 @@ class MWerksCompiler (CCompiler) :
         # into the project.
         if output_filename[-8:] == '.ppc.slb':
             basename = output_filename[:-8]
+        elif output_filename[-11:] == '.carbon.slb':
+            basename = output_filename[:-11]
         else:
             basename = os.path.strip(output_filename)[0]
         projectname = basename + '.mcp'
@@ -122,8 +135,8 @@ class MWerksCompiler (CCompiler) :
         exportname = basename + '.mcp.exp'
         prefixname = 'mwerks_%s_config.h'%basename
         # Create the directories we need
-        distutils.dir_util.mkpath(build_temp, self.verbose, self.dry_run)
-        distutils.dir_util.mkpath(output_dir, self.verbose, self.dry_run)
+        distutils.dir_util.mkpath(build_temp, dry_run=self.dry_run)
+        distutils.dir_util.mkpath(output_dir, dry_run=self.dry_run)
         # And on to filling in the parameters for the project builder
         settings = {}
         settings['mac_exportname'] = exportname
@@ -140,16 +153,16 @@ class MWerksCompiler (CCompiler) :
             if not dirname in sourcefiledirs:
                 sourcefiledirs.append(dirname)
         settings['sources'] = sourcefilenames
+        settings['libraries'] = libraries
         settings['extrasearchdirs'] = sourcefiledirs + include_dirs + library_dirs
         if self.dry_run:
-            print 'CALLING LINKER IN', os.getcwd()
+            print('CALLING LINKER IN', os.getcwd())
             for key, value in settings.items():
-                print '%20.20s %s'%(key, value)
+                print('%20.20s %s'%(key, value))
             return
         # Build the export file
         exportfilename = os.path.join(build_temp, exportname)
-        if self.verbose:
-            print '\tCreate export file', exportfilename
+        log.debug("\tCreate export file %s", exportfilename)
         fp = open(exportfilename, 'w')
         fp.write('%s\n'%export_symbols[0])
         fp.close()
@@ -157,12 +170,12 @@ class MWerksCompiler (CCompiler) :
         if self.__macros:
             prefixfilename = os.path.join(os.getcwd(), os.path.join(build_temp, prefixname))
             fp = open(prefixfilename, 'w')
-            fp.write('#include "mwerks_plugin_config.h"\n')
+            fp.write('#include "mwerks_shcarbon_config.h"\n')
             for name, value in self.__macros:
                 if value is None:
                     fp.write('#define %s\n'%name)
                 else:
-                    fp.write('#define %s "%s"\n'%(name, value))
+                    fp.write('#define %s %s\n'%(name, value))
             fp.close()
             settings['prefixname'] = prefixname
 
@@ -170,8 +183,8 @@ class MWerksCompiler (CCompiler) :
         # because we pass this pathname to CodeWarrior in an AppleEvent, and CW
         # doesn't have a clue about our working directory.
         xmlfilename = os.path.join(os.getcwd(), os.path.join(build_temp, xmlname))
-        if self.verbose:
-            print '\tCreate XML file', xmlfilename
+        log.debug("\tCreate XML file %s", xmlfilename)
+        import mkcwproject
         xmlbuilder = mkcwproject.cwxmlgen.ProjectBuilder(settings)
         xmlbuilder.generate()
         xmldata = settings['tmp_projectxmldata']
@@ -180,14 +193,12 @@ class MWerksCompiler (CCompiler) :
         fp.close()
         # Generate the project. Again a full pathname.
         projectfilename = os.path.join(os.getcwd(), os.path.join(build_temp, projectname))
-        if self.verbose:
-            print '\tCreate project file', projectfilename
+        log.debug('\tCreate project file %s', projectfilename)
         mkcwproject.makeproject(xmlfilename, projectfilename)
         # And build it
-        if self.verbose:
-            print '\tBuild project'
+        log.debug('\tBuild project')
         mkcwproject.buildproject(projectfilename)
-        
+
     def _filename_to_abs(self, filename):
         # Some filenames seem to be unix-like. Convert to Mac names.
 ##        if '/' in filename and ':' in filename:
@@ -196,8 +207,39 @@ class MWerksCompiler (CCompiler) :
 ##           filename = macurl2path(filename)
         filename = distutils.util.convert_path(filename)
         if not os.path.isabs(filename):
-           curdir = os.getcwd()
-           filename = os.path.join(curdir, filename)
-        return filename
-        
-        
+            curdir = os.getcwd()
+            filename = os.path.join(curdir, filename)
+        # Finally remove .. components
+        components = filename.split(':')
+        for i in range(1, len(components)):
+            if components[i] == '..':
+                components[i] = ''
+        return ':'.join(components)
+
+    def library_dir_option (self, dir):
+        """Return the compiler option to add 'dir' to the list of
+        directories searched for libraries.
+        """
+        return # XXXX Not correct...
+
+    def runtime_library_dir_option (self, dir):
+        """Return the compiler option to add 'dir' to the list of
+        directories searched for runtime libraries.
+        """
+        # Nothing needed or Mwerks/Mac.
+        return
+
+    def library_option (self, lib):
+        """Return the compiler option to add 'dir' to the list of libraries
+        linked into the shared library or executable.
+        """
+        return
+
+    def find_library_file (self, dirs, lib, debug=0):
+        """Search the specified list of directories for a static or shared
+        library file 'lib' and return the full path to that file.  If
+        'debug' true, look for a debugging version (if that makes sense on
+        the current platform).  Return None if 'lib' wasn't found in any of
+        the specified directories.
+        """
+        return 0
