@@ -1,20 +1,34 @@
 import ntpath
 import os
-import sys
-from test.test_support import TestFailed
-from test import test_support, test_genericpath
+from test.support import verbose, TestFailed
+import test.support as support
 import unittest
 
-def tester0(fn, wantResult):
-    gotResult = eval(fn)
-    if wantResult != gotResult:
-        raise TestFailed, "%s should return: %r but returned: %r" \
-              %(fn, wantResult, gotResult)
 
 def tester(fn, wantResult):
     fn = fn.replace("\\", "\\\\")
-    tester0(fn, wantResult)
+    gotResult = eval(fn)
+    if wantResult != gotResult:
+        raise TestFailed("%s should return: %s but returned: %s" \
+              %(str(fn), str(wantResult), str(gotResult)))
 
+    # then with bytes
+    fn = fn.replace("('", "(b'")
+    fn = fn.replace('("', '(b"')
+    fn = fn.replace("['", "[b'")
+    fn = fn.replace('["', '[b"')
+    fn = fn.replace(", '", ", b'")
+    fn = fn.replace(', "', ', b"')
+    gotResult = eval(fn)
+    if isinstance(wantResult, str):
+        wantResult = wantResult.encode('ascii')
+    elif isinstance(wantResult, tuple):
+        wantResult = tuple(r.encode('ascii') for r in wantResult)
+
+    gotResult = eval(fn)
+    if wantResult != gotResult:
+        raise TestFailed("%s should return: %s but returned: %s" \
+              %(str(fn), str(wantResult), repr(gotResult)))
 
 class TestNtpath(unittest.TestCase):
     def test_splitext(self):
@@ -36,24 +50,10 @@ class TestNtpath(unittest.TestCase):
                ('c:', '/foo/bar'))
 
     def test_splitunc(self):
-        tester('ntpath.splitunc("c:\\foo\\bar")',
-               ('', 'c:\\foo\\bar'))
-        tester('ntpath.splitunc("c:/foo/bar")',
-               ('', 'c:/foo/bar'))
         tester('ntpath.splitunc("\\\\conky\\mountpoint\\foo\\bar")',
                ('\\\\conky\\mountpoint', '\\foo\\bar'))
         tester('ntpath.splitunc("//conky/mountpoint/foo/bar")',
                ('//conky/mountpoint', '/foo/bar'))
-        tester('ntpath.splitunc("\\\\\\conky\\mountpoint\\foo\\bar")',
-               ('', '\\\\\\conky\\mountpoint\\foo\\bar'))
-        tester('ntpath.splitunc("///conky/mountpoint/foo/bar")',
-               ('', '///conky/mountpoint/foo/bar'))
-        tester('ntpath.splitunc("\\\\conky\\\\mountpoint\\foo\\bar")',
-               ('', '\\\\conky\\\\mountpoint\\foo\\bar'))
-        tester('ntpath.splitunc("//conky//mountpoint/foo/bar")',
-               ('', '//conky//mountpoint/foo/bar'))
-        self.assertEqual(ntpath.splitunc(u'//conky/MOUNTPO\u0130NT/foo/bar'),
-                         (u'//conky/MOUNTPO\u0130NT', u'/foo/bar'))
 
     def test_split(self):
         tester('ntpath.split("c:\\foo\\bar")', ('c:\\foo', 'bar'))
@@ -88,7 +88,10 @@ class TestNtpath(unittest.TestCase):
         tester('ntpath.join("/a")', '/a')
         tester('ntpath.join("\\a")', '\\a')
         tester('ntpath.join("a:")', 'a:')
+        tester('ntpath.join("a:", "b")', 'a:b')
+        tester('ntpath.join("a:", "/b")', 'a:/b')
         tester('ntpath.join("a:", "\\b")', 'a:\\b')
+        tester('ntpath.join("a", "/b")', '/b')
         tester('ntpath.join("a", "\\b")', '\\b')
         tester('ntpath.join("a", "b", "c")', 'a\\b\\c')
         tester('ntpath.join("a\\", "b", "c")', 'a\\b\\c')
@@ -96,46 +99,22 @@ class TestNtpath(unittest.TestCase):
         tester('ntpath.join("a", "b", "\\c")', '\\c')
         tester('ntpath.join("d:\\", "\\pleep")', 'd:\\pleep')
         tester('ntpath.join("d:\\", "a", "b")', 'd:\\a\\b')
+        tester("ntpath.join('c:', '/a')", 'c:/a')
+        tester("ntpath.join('c:/', '/a')", 'c:/a')
+        tester("ntpath.join('c:/a', '/b')", '/b')
+        tester("ntpath.join('c:', 'd:/')", 'd:/')
+        tester("ntpath.join('c:/', 'd:/')", 'd:/')
+        tester("ntpath.join('c:/', 'd:/a/b')", 'd:/a/b')
 
+        tester("ntpath.join('')", '')
+        tester("ntpath.join('', '', '', '', '')", '')
+        tester("ntpath.join('a')", 'a')
         tester("ntpath.join('', 'a')", 'a')
         tester("ntpath.join('', '', '', '', 'a')", 'a')
         tester("ntpath.join('a', '')", 'a\\')
         tester("ntpath.join('a', '', '', '', '')", 'a\\')
         tester("ntpath.join('a\\', '')", 'a\\')
         tester("ntpath.join('a\\', '', '', '', '')", 'a\\')
-        tester("ntpath.join('a/', '')", 'a/')
-
-        tester("ntpath.join('a/b', 'x/y')", 'a/b\\x/y')
-        tester("ntpath.join('/a/b', 'x/y')", '/a/b\\x/y')
-        tester("ntpath.join('/a/b/', 'x/y')", '/a/b/x/y')
-        tester("ntpath.join('c:', 'x/y')", 'c:x/y')
-        tester("ntpath.join('c:a/b', 'x/y')", 'c:a/b\\x/y')
-        tester("ntpath.join('c:a/b/', 'x/y')", 'c:a/b/x/y')
-        tester("ntpath.join('c:/', 'x/y')", 'c:/x/y')
-        tester("ntpath.join('c:/a/b', 'x/y')", 'c:/a/b\\x/y')
-        tester("ntpath.join('c:/a/b/', 'x/y')", 'c:/a/b/x/y')
-        #tester("ntpath.join('//computer/share', 'x/y')", '//computer/share\\x/y')
-        #tester("ntpath.join('//computer/share/', 'x/y')", '//computer/share/x/y')
-        #tester("ntpath.join('//computer/share/a/b', 'x/y')", '//computer/share/a/b\\x/y')
-
-        tester("ntpath.join('a/b', '/x/y')", '/x/y')
-        tester("ntpath.join('/a/b', '/x/y')", '/x/y')
-        tester("ntpath.join('c:', '/x/y')", 'c:/x/y')
-        tester("ntpath.join('c:a/b', '/x/y')", 'c:/x/y')
-        tester("ntpath.join('c:/', '/x/y')", 'c:/x/y')
-        tester("ntpath.join('c:/a/b', '/x/y')", 'c:/x/y')
-        #tester("ntpath.join('//computer/share', '/x/y')", '//computer/share/x/y')
-        #tester("ntpath.join('//computer/share/', '/x/y')", '//computer/share/x/y')
-        #tester("ntpath.join('//computer/share/a', '/x/y')", '//computer/share/x/y')
-
-        tester("ntpath.join('c:', 'C:x/y')", 'C:x/y')
-        tester("ntpath.join('c:a/b', 'C:x/y')", 'C:a/b\\x/y')
-        tester("ntpath.join('c:/', 'C:x/y')", 'C:/x/y')
-        tester("ntpath.join('c:/a/b', 'C:x/y')", 'C:/a/b\\x/y')
-
-        for x in ('', 'a/b', '/a/b', 'c:', 'c:a/b', 'c:/', 'c:/a/b'):
-            for y in ('d:', 'd:x/y', 'd:/', 'd:/x/y'):
-                tester("ntpath.join(%r, %r)" % (x, y), y)
 
     def test_normpath(self):
         tester("ntpath.normpath('A//////././//.//B')", r'A\B')
@@ -161,21 +140,20 @@ class TestNtpath(unittest.TestCase):
         tester("ntpath.normpath('C:////a/b')", r'C:\a\b')
         tester("ntpath.normpath('//machine/share//a/b')", r'\\machine\share\a\b')
 
-        tester("ntpath.normpath('\\\\.\\NUL')", r'\\.\NUL')
-        tester("ntpath.normpath('\\\\?\\D:/XY\\Z')", r'\\?\D:/XY\Z')
-
     def test_expandvars(self):
-        with test_support.EnvironmentVarGuard() as env:
-            env.clear()
-            env["foo"] = "bar"
-            env["{foo"] = "baz1"
-            env["{foo}"] = "baz2"
+        oldenv = os.environ.copy()
+        try:
+            os.environ.clear()
+            os.environ["foo"] = "bar"
+            os.environ["{foo"] = "baz1"
+            os.environ["{foo}"] = "baz2"
             tester('ntpath.expandvars("foo")', "foo")
             tester('ntpath.expandvars("$foo bar")', "bar bar")
             tester('ntpath.expandvars("${foo}bar")', "barbar")
             tester('ntpath.expandvars("$[foo]bar")', "$[foo]bar")
             tester('ntpath.expandvars("$bar bar")', "$bar bar")
             tester('ntpath.expandvars("$?bar")', "$?bar")
+            tester('ntpath.expandvars("${foo}bar")', "barbar")
             tester('ntpath.expandvars("$foo}bar")', "bar}bar")
             tester('ntpath.expandvars("${foo")', "${foo")
             tester('ntpath.expandvars("${{foo}}")', "baz1}")
@@ -188,30 +166,9 @@ class TestNtpath(unittest.TestCase):
             tester('ntpath.expandvars("%?bar%")', "%?bar%")
             tester('ntpath.expandvars("%foo%%bar")', "bar%bar")
             tester('ntpath.expandvars("\'%foo%\'%bar")', "\'%foo%\'%bar")
-
-    @unittest.skipUnless(test_support.FS_NONASCII, 'need test_support.FS_NONASCII')
-    def test_expandvars_nonascii(self):
-        encoding = sys.getfilesystemencoding()
-        def check(value, expected):
-            tester0("ntpath.expandvars(%r)" % value, expected)
-            tester0("ntpath.expandvars(%r)" % value.decode(encoding),
-                    expected.decode(encoding))
-        with test_support.EnvironmentVarGuard() as env:
-            env.clear()
-            unonascii = test_support.FS_NONASCII
-            snonascii = unonascii.encode(encoding)
-            env['spam'] = snonascii
-            env[snonascii] = 'ham' + snonascii
-            check('$spam bar', '%s bar' % snonascii)
-            check('$%s bar' % snonascii, '$%s bar' % snonascii)
-            check('${spam}bar', '%sbar' % snonascii)
-            check('${%s}bar' % snonascii, 'ham%sbar' % snonascii)
-            check('$spam}bar', '%s}bar' % snonascii)
-            check('$%s}bar' % snonascii, '$%s}bar' % snonascii)
-            check('%spam% bar', '%s bar' % snonascii)
-            check('%{}% bar'.format(snonascii), 'ham%s bar' % snonascii)
-            check('%spam%bar', '%sbar' % snonascii)
-            check('%{}%bar'.format(snonascii), 'ham%sbar' % snonascii)
+        finally:
+            os.environ.clear()
+            os.environ.update(oldenv)
 
     def test_abspath(self):
         # ntpath.abspath() can only be used on a system with the "nt" module
@@ -219,12 +176,10 @@ class TestNtpath(unittest.TestCase):
         # the rest of the tests for the ntpath module to be run to completion
         # on any platform, since most of the module is intended to be usable
         # from any platform.
-        # XXX this needs more tests
         try:
             import nt
         except ImportError:
-            # check that the function is there even if we are not on Windows
-            ntpath.abspath
+            pass
         else:
             tester('ntpath.abspath("C:\\")', "C:\\")
 
@@ -239,25 +194,10 @@ class TestNtpath(unittest.TestCase):
         tester('ntpath.relpath("a", "b/c")', '..\\..\\a')
         tester('ntpath.relpath("//conky/mountpoint/a", "//conky/mountpoint/b/c")', '..\\..\\a')
         tester('ntpath.relpath("a", "a")', '.')
-        tester('ntpath.relpath("/foo/bar/bat", "/x/y/z")', '..\\..\\..\\foo\\bar\\bat')
-        tester('ntpath.relpath("/foo/bar/bat", "/foo/bar")', 'bat')
-        tester('ntpath.relpath("/foo/bar/bat", "/")', 'foo\\bar\\bat')
-        tester('ntpath.relpath("/", "/foo/bar/bat")', '..\\..\\..')
-        tester('ntpath.relpath("/foo/bar/bat", "/x")', '..\\foo\\bar\\bat')
-        tester('ntpath.relpath("/x", "/foo/bar/bat")', '..\\..\\..\\x')
-        tester('ntpath.relpath("/", "/")', '.')
-        tester('ntpath.relpath("/a", "/a")', '.')
-        tester('ntpath.relpath("/a/b", "/a/b")', '.')
-        tester('ntpath.relpath("c:/foo", "C:/FOO")', '.')
-
-
-class NtCommonTest(test_genericpath.CommonTest):
-    pathmodule = ntpath
-    attributes = ['relpath', 'splitunc']
 
 
 def test_main():
-    test_support.run_unittest(TestNtpath, NtCommonTest)
+    support.run_unittest(TestNtpath)
 
 
 if __name__ == "__main__":
