@@ -5,8 +5,8 @@
 # complains several times about module random having no attribute
 # randrange, and then Python hangs.
 
-import thread
-from test_support import verbose, TestSkipped
+import _thread as thread
+from test.support import verbose, TestSkipped, TestFailed
 
 critical_section = thread.allocate_lock()
 done = thread.allocate_lock()
@@ -17,9 +17,29 @@ def task():
     x = random.randrange(1, 3)
     critical_section.acquire()
     N -= 1
-    if N == 0:
-        done.release()
+    # Must release critical_section before releasing done, else the main
+    # thread can exit and set critical_section to None as part of global
+    # teardown; then critical_section.release() raises AttributeError.
+    finished = N == 0
     critical_section.release()
+    if finished:
+        done.release()
+
+def test_import_hangers():
+    import sys
+    if verbose:
+        print("testing import hangers ...", end=' ')
+
+    import test.threaded_import_hangers
+    try:
+        if test.threaded_import_hangers.errors:
+            raise TestFailed(test.threaded_import_hangers.errors)
+        elif verbose:
+            print("OK.")
+    finally:
+        # In case this test is run again, make sure the helper module
+        # gets loaded from scratch again.
+        del sys.modules['test.threaded_import_hangers']
 
 # Tricky:  When regrtest imports this module, the thread running regrtest
 # grabs the import lock and won't let go of it until this module returns.
@@ -41,12 +61,15 @@ def test_main():        # magic name!  see above
     done.acquire()
     for N in (20, 50) * 3:
         if verbose:
-            print "Trying", N, "threads ...",
+            print("Trying", N, "threads ...", end=' ')
         for i in range(N):
             thread.start_new_thread(task, ())
         done.acquire()
         if verbose:
-            print "OK."
+            print("OK.")
+    done.release()
+
+    test_import_hangers()
 
 if __name__ == "__main__":
     test_main()
